$pi = 3.14159;


function eulerToAxis(%euler)
{
	%euler = VectorScale(%euler,$pi / 180);
	%matrix = MatrixCreateFromEuler(%euler);
	return getWords(%matrix,3,6);
}

function axisToEuler(%axis)
{
	%angleOver2 = getWord(%axis,3) * 0.5;
	%angleOver2 = -%angleOver2;
	%sinThetaOver2 = mSin(%afngleOver2);
	%cosThetaOver2 = mCos(%angleOver2);
	%q0 = %cosThetaOver2;
	%q1 = getWord(%axis,0) * %sinThetaOver2;
	%q2 = getWord(%axis,1) * %sinThetaOver2;
	%q3 = getWord(%axis,2) * %sinThetaOver2;
	%q0q0 = %q0 * %q0;
	%q1q2 = %q1 * %q2;
	%q0q3 = %q0 * %q3;
	%q1q3 = %q1 * %q3;
	%q0q2 = %q0 * %q2;
	%q2q2 = %q2 * %q2;
	%q2q3 = %q2 * %q3;
	%q0q1 = %q0 * %q1;
	%q3q3 = %q3 * %q3;
	%m13 = 2.0 * (%q1q3 - %q0q2);
	%m21 = 2.0 * (%q1q2 - %q0q3);
	%m22 = 2.0 * %q0q0 - 1.0 + 2.0 * %q2q2;
	%m23 = 2.0 * (%q2q3 + %q0q1);
	%m33 = 2.0 * %q0q0 - 1.0 + 2.0 * %q3q3;
	return mRadToDeg(mAsin(%m23)) SPC mRadToDeg(mAtan(-%m13, %m33)) SPC mRadToDeg(mAtan(-%m21, %m22));
}

function getdistance(%a, %b)
{
	if(!isobject(%a) || !isobject(%b)) return -1;
	%t = %a.gettransform();
	%ax = getword(%t, 0);
	%ay = getword(%t, 1);
	%az = getword(%t, 2);
	%t = %b.gettransform();
	%bx = getword(%t, 0);
	%by = getword(%t, 1);
	%bz = getword(%t, 2);
	%x = %ax - %bx;
	%y = %ay - %by;
	%z = %az - %bz;
	%dist = msqrt((%x * %x) + (%y * %y) + (%z * %z));
	return %dist;
}

function pointdistance(%a, %b)
{
	%ax = getword(%a, 0);
	%ay = getword(%a, 1);
	%az = getword(%a, 2);
	%bx = getword(%b, 0);
	%by = getword(%b, 1);
	%bz = getword(%b, 2);
	%x = %ax - %bx;
	%y = %ay - %by;
	%z = %az - %bz;
	%dist = msqrt((%x * %x) + (%y * %y) + (%z * %z));
	return %dist;
}

function ScriptObject::TeamIsDead(%minigame, %team)
{
	for(%i = 0; %i < %minigame.numMembers; %i++) {
		if(%minigame.member[%i].team == %team && !%minigame.member[%i].dead) return false;
	}
	return true;
}

function getInaccuratePoint(%origin, %target, %maxTheta) //takes a shooter's eyepoint and a target, and offsets it a random amount up to a max angle theta.
{
	%relTargetVector = vectoradd(%target, vectorScale(%origin, -1));
	%relNormalTV = VectorNormalize(%relTargetVector);
	%distance = pointDistance(%origin, %target);
	%theta = (getrandom(0, 100) / 100) * %maxTheta;
	
	%vecLatComp = msin(%theta);
		//This is the absolute distance in a single dimension perpendicular to %relTargetVector from the target;
		
	%pointAngle = getrandom(0, 200 * $Pi) / 100;
		//%pointangle rotates %vecLatComp in a circle around the target starting at pure +Z and rotating clockwise.

	%vecRotLatCompRelX = %vecLatComp * msin(%pointAngle);
	%vecRotLatCompRelY = %vecLatComp * mcos(%pointAngle);
		//relative X and relative Y are X/Y on a sphere's interior with the shooter at the origin, with the "origin" on the sphere being the target.
		//however, the sphere has been flattened to a plane perpendicular to the shooter's X/Y vector.
		//we know the circle on this plane to have the radius of VecLateralComp.
		//The "leaning in" from the actual sphere shape occurs with %vecAdjustedDistComp below, which will move us inward (toward shooter) properly relative to our arc length.
	
		
	%playerXYRot = mAtan(getword(%relNormalTV, 1), getword(%relNormalTV, 0));
	%relXAbsXFactor = msin(%playerXYRot) * -1;
	%relXAbsYFactor = mcos(%playerXYRot) * -1;
	
	%vecAddXYExtraDistComp = %vecRotLatCompRelY * mAsin(getword(%relNormalTV, 2)) * -1;
		//at a non-flat angle, the extra distance component is how much to subtract the final X/Y distance. Note that at a positive Z component, the circle half centered on zero or 2pi is closer.
	
	%vecAbsX = %relXAbsXFactor * (%vecRotLatCompRelX + %vecAddXYExtraDistComp);
	%vecAbsY = %relXAbsXFactor * (%vecRotLatCompRelX + %vecAddXYExtraDistComp);
	%vecAbsZ = %VecRotLatCompRelY * mAcos(getword(%relNormalTV, 2));
		//at a flat angle, we just use relative Y for absolute Z; otherwise it's reduced

	%finalAbsVec = %vecAbsX SPC %vecAbsY SPC %vecAbsZ;

	%vecAdjustedDistComp = %distance * mcos(%theta);
	%distFactor = %vecAdjustedDistComp / %distance;
		//This effects the absolute distance towards the shooter due to it being an arc of a sphere, not a line on a plane
	
	%RelativeAimPoint = vectorAdd(%finalAbsVec, vectorScale(%relTargetVector, %distfactor));
		//final aim point for a player at (0,0,0)
		
	%AbsoluteAimPoint = vectorAdd(%RelativeAimPoint, %origin);
	
	return %AbsoluteAimPoint;
}